@@ -50,7 +50,7 @@ has type =>
 	required => 0,
 );
 
-our $VERSION = '2.12';
+our $VERSION = '3.00';
 
 # -----------------------------------------------
 
@@ -79,7 +79,7 @@ sub columns
 		return [sort{$a cmp $b} keys %{$$info{$table}{columns} }];
 	}
 
-}	# End of columns.
+} # End of columns.
 
 # -----------------------------------------------
 # Warning: This is a function, not a method.
@@ -180,6 +180,8 @@ sub _info
 
 	for $table_name (@table_name)
 	{
+		$$info{$table_name}{foreign_keys} = [];
+
 		for $foreign_table (grep{! /^$table_name$/} @table_name)
 		{
 			if ($vendor eq 'SQLITE')
@@ -188,7 +190,7 @@ sub _info
 				{
 					next if ($$row[2] ne $table_name);
 
-					$$info{$table_name}{foreign_keys}{$foreign_table} =
+					push @{$$info{$table_name}{foreign_keys} },
 					{
 						DEFERABILITY      => undef,
 						DELETE_RULE       => $referential_action{$$row[6]},
@@ -203,7 +205,7 @@ sub _info
 						UK_DATA_TYPE      => undef,
 						UK_NAME           => undef,
 						UK_TABLE_CAT      => undef,
-						UK_TABLE_NAME     => $table_name,
+						UK_TABLE_NAME     => $$row[2],
 						UK_TABLE_SCHEM    => undef,
 						UNIQUE_OR_PRIMARY => undef,
 						UPDATE_RULE       => $referential_action{$$row[5]},
@@ -214,16 +216,17 @@ sub _info
 			{
 				$table_sth = $self -> dbh -> foreign_key_info($self -> catalog, $self -> schema, $table_name, $self -> catalog, $self -> schema, $foreign_table) || next;
 
-				if ($vendor eq 'MySQL')
+				if ($vendor eq 'MYSQL')
 				{
 					my($hashref) = $table_sth->fetchall_hashref(['PKTABLE_NAME']);
-					$$info{$table_name}{foreign_keys}{$foreign_table} = $$hashref{$table_name} if ($$hashref{$table_name});
+
+					push @{$$info{$table_name}{foreign_keys} }, $$hashref{$table_name} if ($$hashref{$table_name});
 				}
 				else
 				{
 					for $column_data (@{$table_sth -> fetchall_arrayref({})})
 					{
-						$$info{$table_name}{foreign_keys}{$foreign_table} = {%$column_data};
+						push @{$$info{$table_name}{foreign_keys} }, {%$column_data};
 					}
 				}
 			}
@@ -232,7 +235,7 @@ sub _info
 
 	$self -> info($info);
 
-}	# End of _info.
+} # End of _info.
 
 # -----------------------------------------------
 
@@ -244,7 +247,7 @@ sub refresh
 
 	return $self -> info;
 
-}	# End of refresh.
+} # End of refresh.
 
 # -----------------------------------------------
 
@@ -254,14 +257,12 @@ sub tables
 
 	return [sort keys %{$self -> info}];
 
-}	# End of tables.
+} # End of tables.
 
 # -----------------------------------------------
 
 1;
 
-__END__
-
 =head1 NAME
 
 DBIx::Admin::TableInfo - A wrapper for all of table_info(), column_info(), *_key_info()
@@ -294,7 +295,7 @@ Also, for Postgres, you can set DBI_SCHEMA to a list of schemas, e.g. when proce
 
 For details, see L<http://blogs.perl.org/users/ron_savage/2013/03/graphviz2-and-the-dread-musicbrainz-db.html>.
 
-See also xt/author/mysql.fk.pl and xt/author/fk.t.
+See also xt/author/fk.t, xt/author/mysql.fk.pl and xt/author/person.spouse.t.
 
 =head1 Description
 
@@ -314,17 +315,11 @@ It is a convenient wrapper around all of these DBI methods:
 
 =back
 
-Warnings:
-
 =over 4
 
 =item o MySQL
 
-=over 4
-
-=item o New Notes
-
-I am testing V 2.04 of this module with MySql V 5.0.51a and DBD::mysql V 4.014.
+Warning:
 
 To get foreign key information in the output, the create table statement has to:
 
@@ -352,34 +347,6 @@ has to, for MySql, look like:
 
 	create table designs (...) engine=innodb
 
-You have been warned.
-
-=back
-
-=item o Old Notes
-
-The MySQL client C<DBD::mysql> V 3.0002 does not support C<primary_key_info()>,
-so this module emulates it by stockpiling a list of columns which have the
-attribute 'mysql_is_pri_key' set.
-
-The problem with this is that if a primary key consists of more than 1 column,
-C<DBD::mysql> does not indicate the order of these columns within the key, so
-this module pretends that they are in the same order as the order of columns
-returned by the call to C<column_info()>.
-
-Likewise, C<DBD::mysql> does not support C<foreign_key_info()>, so in the case
-of MySQL, nothing is reported for foreign keys.
-
-For MySQL V 5.0.18, section 14.2.6.4 of the manual says that for InnoDB tables,
-the SQL "show table status from 'db name' like 'table name'" will display the foreign
-key info in the column called 'Comment', but this is simply not true. The 'Comment'
-column contains a string such as 'InnoDB free: 4096 kB'.
-
-Likewise, the SQL "show create table 'table name'" reveals than MySQL does not
-preserve 'create table' clauses such as 'references other_table(other_column)'.
-
-So, at the moment, I see no way of displaying foreign key information under MySQL.
-
 =back
 
 =item o Oracle
@@ -388,8 +355,6 @@ See the L</FAQ> for which tables are ignored under Oracle.
 
 =item o Postgres
 
-I am testing V 2.04 of this module with Postgres V 08.03.1100 and DBD::Pg V 2.17.1.
-
 The latter now takes '%' as the value of the 'table' parameter to new(), whereas
 older versions of DBD::Pg required 'table' to be set to 'table'.
 
@@ -397,8 +362,6 @@ See the L</FAQ> for which tables are ignored under Postgres.
 
 =item o SQLite
 
-I am testing V 2.04 of this module with SQLite V 3.6.22 and DBD::SQLite V 1.29.
-
 See the L</FAQ> for which tables are ignored under SQLite.
 
 =back
@@ -588,20 +551,18 @@ For the attributes of the tables, there are no more levels in the hash ref.
 
 =back
 
-=item o Third level, after 'foreign_keys': The keys are the names of tables
+=item o Third level, after 'foreign_keys': An arrayref contains the details (if any)
 
-These tables have foreign keys which point to the current table.
+But beware slightly differing spellings depending on the database server. This is documented in
+L<https://metacpan.org/pod/DBI#foreign_key_info>. Look closely at the usage of the '_' character.
 
-	my($foreign_keys) = $$info{$table_name}{foreign_keys};
+	my($vendor) = uc $dbh -> get_info(17); # SQL_DBMS_NAME.
 
-	for $foreign_table (sort keys %$foreign_keys)
+	for $item (@{$$info{$table_name}{foreign_keys} })
 	{
-		$foreign_key = $$foreign_keys{$foreign_table};
+		# Get the name of the table pointed to.
 
-		for $attribute (sort keys %$foreign_key)
-		{
-			Use...
-		}
+		$primary_table = ($vendor eq 'MYSQL') ? $$item{PKTABLE_NAME} : $$item{UK_TABLE_NAME};
 	}
 
 =item o Third level, after 'primary_keys': The keys are the names of columns
@@ -707,18 +668,20 @@ See the examples/ directory in the distro.
 
 =head2 Which versions of the servers did you test?
 
-	Versions as at 2014-03-07
-	+----------|------------+
-	|  Vendor  |     V      |
-	+----------|------------+
-	|  MariaDB |   5.5.36   |
-	+----------|------------+
-	|  Oracle  | 10.2.0.1.0 | (Not tested for years)
-	+----------|------------+
-	| Postgres |   9.1.12   |
-	+----------|------------+
-	|  SQLite  |   3.7.17   |
-	+----------|------------+
+	Versions as at 2014-08-06:
+	+----------|-------------+
+	|  Vendor  |      V      |
+	+----------|-------------+
+	|  MariaDB |   5.5.38    |
+	+----------|-------------+
+	|  Oracle  | 10.2.0.1.0  | (Not tested for years)
+	+----------|-------------+
+	| Postgres |    9.1.3    |
+	+----------|-------------+
+	|  SQLite  |   3.8.4.1   |
+	+----------|-------------+
+
+But see these L<warnings|https://metacpan.org/pod/DBIx::Admin::TableInfo#Description> when using MySQL/MariaDB.
 
 =head2 Which tables are ignored for which databases?
 
@@ -730,15 +693,17 @@ Here is the code which skips some tables:
 
 =head2 How do I identify foreign keys?
 
+Note: The table names here come from xt/author/person.spouse.t.
+
 See L<DBIx::Admin::CreateTable/FAQ> for database server-specific create statements to activate foreign keys.
 
 Then try:
 
 	my($info) = DBIx::Admin::TableInfo -> new(dbh => $dbh) -> info;
 
-	print Data::Dumper::Concise::Dumper($$info{one}{foreign_keys}), "\n";
+	print Data::Dumper::Concise::Dumper($$info{people}{foreign_keys}), "\n";
 
-Output follows. Each is a hashref with the keys being the names of tables (in this case 'two') pointing to table 'one'.
+Output follows.
 
 But beware slightly differing spellings depending on the database server. This is documented in
 L<https://metacpan.org/pod/DBI#foreign_key_info>. Look closely at the usage of the '_' character.
@@ -747,85 +712,136 @@ L<https://metacpan.org/pod/DBI#foreign_key_info>. Look closely at the usage of t
 
 =item o MySQL
 
-	two => {
-		DEFERABILITY => undef,
-		DELETE_RULE => undef,
-		FKCOLUMN_NAME => "one_id",
-		FKTABLE_CAT => "def",
-		FKTABLE_NAME => "two",
-		FKTABLE_SCHEM => "testdb",
-		FK_NAME => "two_ibfk_1",
-		KEY_SEQ => 1,
-		PKCOLUMN_NAME => "id",
-		PKTABLE_CAT => undef,
-		PKTABLE_NAME => "one",
-		PKTABLE_SCHEM => "testdb",
-		PK_NAME => undef,
-		UNIQUE_OR_PRIMARY => undef,
-		UPDATE_RULE => undef
-	}
+	[
+	  {
+	    DEFERABILITY => undef,
+	    DELETE_RULE => undef,
+	    FKCOLUMN_NAME => "spouse_id",
+	    FKTABLE_CAT => "def",
+	    FKTABLE_NAME => "spouses",
+	    FKTABLE_SCHEM => "testdb",
+	    FK_NAME => "spouses_ibfk_2",
+	    KEY_SEQ => 1,
+	    PKCOLUMN_NAME => "id",
+	    PKTABLE_CAT => undef,
+	    PKTABLE_NAME => "people",
+	    PKTABLE_SCHEM => "testdb",
+	    PK_NAME => undef,
+	    UNIQUE_OR_PRIMARY => undef,
+	    UPDATE_RULE => undef
+	  }
+	]
+
+Yes, there is just 1 element in this arrayref. MySQL can sliently drop an index if another index can be used.
 
 =item o Postgres
 
-	two => {
-		DEFERABILITY => 7,
-		DELETE_RULE => 3,
-		FK_COLUMN_NAME => "one_id",
-		FK_DATA_TYPE => "int4",
-		FK_NAME => "two_one_id_fkey",
-		FK_TABLE_CAT => undef,
-		FK_TABLE_NAME => "two",
-		FK_TABLE_SCHEM => "public",
-		ORDINAL_POSITION => 1,
-		UK_COLUMN_NAME => "id",
-		UK_DATA_TYPE => "int4",
-		UK_NAME => "one_pkey",
-		UK_TABLE_CAT => undef,
-		UK_TABLE_NAME => "one",
-		UK_TABLE_SCHEM => "public",
-		UNIQUE_OR_PRIMARY => "PRIMARY",
-		UPDATE_RULE => 3
-	}
+	[
+	  {
+	    DEFERABILITY => 7,
+	    DELETE_RULE => 3,
+	    FK_COLUMN_NAME => "person_id",
+	    FK_DATA_TYPE => "int4",
+	    FK_NAME => "spouses_person_id_fkey",
+	    FK_TABLE_CAT => undef,
+	    FK_TABLE_NAME => "spouses",
+	    FK_TABLE_SCHEM => "public",
+	    ORDINAL_POSITION => 1,
+	    UK_COLUMN_NAME => "id",
+	    UK_DATA_TYPE => "int4",
+	    UK_NAME => "people_pkey",
+	    UK_TABLE_CAT => undef,
+	    UK_TABLE_NAME => "people",
+	    UK_TABLE_SCHEM => "public",
+	    UNIQUE_OR_PRIMARY => "PRIMARY",
+	    UPDATE_RULE => 3
+	  },
+	  {
+	    DEFERABILITY => 7,
+	    DELETE_RULE => 3,
+	    FK_COLUMN_NAME => "spouse_id",
+	    FK_DATA_TYPE => "int4",
+	    FK_NAME => "spouses_spouse_id_fkey",
+	    FK_TABLE_CAT => undef,
+	    FK_TABLE_NAME => "spouses",
+	    FK_TABLE_SCHEM => "public",
+	    ORDINAL_POSITION => 1,
+	    UK_COLUMN_NAME => "id",
+	    UK_DATA_TYPE => "int4",
+	    UK_NAME => "people_pkey",
+	    UK_TABLE_CAT => undef,
+	    UK_TABLE_NAME => "people",
+	    UK_TABLE_SCHEM => "public",
+	    UNIQUE_OR_PRIMARY => "PRIMARY",
+	    UPDATE_RULE => 3
+	  }
+	]
 
 =item o SQLite
 
-	two => {
-		DEFERABILITY => undef,
-		DELETE_RULE => 3,
-		FK_COLUMN_NAME => "one_id",
-		FK_DATA_TYPE => undef,
-		FK_NAME => undef,
-		FK_TABLE_CAT => undef,
-		FK_TABLE_NAME => "two",
-		FK_TABLE_SCHEM => undef,
-		ORDINAL_POSITION => 0,
-		UK_COLUMN_NAME => "id",
-		UK_DATA_TYPE => undef,
-		UK_NAME => undef,
-		UK_TABLE_CAT => undef,
-		UK_TABLE_NAME => "one",
-		UK_TABLE_SCHEM => undef,
-		UNIQUE_OR_PRIMARY => undef,
-		UPDATE_RULE => 3
-	}
+	[
+	  {
+	    DEFERABILITY => undef,
+	    DELETE_RULE => 3,
+	    FK_COLUMN_NAME => "spouse_id",
+	    FK_DATA_TYPE => undef,
+	    FK_NAME => undef,
+	    FK_TABLE_CAT => undef,
+	    FK_TABLE_NAME => "spouses",
+	    FK_TABLE_SCHEM => undef,
+	    ORDINAL_POSITION => 0,
+	    UK_COLUMN_NAME => "id",
+	    UK_DATA_TYPE => undef,
+	    UK_NAME => undef,
+	    UK_TABLE_CAT => undef,
+	    UK_TABLE_NAME => "people",
+	    UK_TABLE_SCHEM => undef,
+	    UNIQUE_OR_PRIMARY => undef,
+	    UPDATE_RULE => 3
+	  },
+	  {
+	    DEFERABILITY => undef,
+	    DELETE_RULE => 3,
+	    FK_COLUMN_NAME => "person_id",
+	    FK_DATA_TYPE => undef,
+	    FK_NAME => undef,
+	    FK_TABLE_CAT => undef,
+	    FK_TABLE_NAME => "spouses",
+	    FK_TABLE_SCHEM => undef,
+	    ORDINAL_POSITION => 0,
+	    UK_COLUMN_NAME => "id",
+	    UK_DATA_TYPE => undef,
+	    UK_NAME => undef,
+	    UK_TABLE_CAT => undef,
+	    UK_TABLE_NAME => "people",
+	    UK_TABLE_SCHEM => undef,
+	    UNIQUE_OR_PRIMARY => undef,
+	    UPDATE_RULE => 3
+	  }
+	]
 
 =back
 
 You can also play with xt/author/fk.t and xt/author/dsn.ini (especially the 'active' option).
 
+fk.t does not delete the tables as it exits. This is so xt/author/mysql.fk.pl has something to play with.
+
+See also xt/author/person.spouse.t.
+
 =head2 Does DBIx::Admin::TableInfo work with SQLite databases?
 
-Yes. As of V 2.08, this module uses SQLite's "pragma foreign_key_list($table_name)" to emulate L<DBI>'s
-$dbh -> foreign_key_info(...).
+Yes. As of V 2.08, this module uses the SQLite code "pragma foreign_key_list($table_name)" to emulate the L<DBI>
+call to foreign_key_info(...).
 
 =head2 What is returned by the SQLite "pragma foreign_key_list($table_name)" call?
 
-	Fields returned are:
+An arrayref is returned. Indexes and their interpretations:
+
 	0: COUNT   (0, 1, ...)
 	1: KEY_SEQ (0, or column # (1, 2, ...) within multi-column key)
-	2: FKTABLE_NAME
-	3: PKCOLUMN_NAME
-	4: FKCOLUMN_NAME
+	2: PK_TABLE_NAME
+	3: FK_COLUMN_NAME
+	4: PK_COLUMN_NAME
 	5: UPDATE_RULE
 	6: DELETE_RULE
 	7: 'NONE' (Constant string)
@@ -834,6 +850,8 @@ As these are stored in an arrayref, I use $$row[$i] just below to refer to the e
 
 =head2 How are these values mapped into the output?
 
+See also the next point.
+
 	my(%referential_action) =
 	(
 		'CASCADE'     => 0,
@@ -852,14 +870,14 @@ The hashref returned for foreign keys contains these key-value pairs:
 		FK_DATA_TYPE      => undef,
 		FK_NAME           => undef,
 		FK_TABLE_CAT      => undef,
-		FK_TABLE_NAME     => $foreign_table,
+		FK_TABLE_NAME     => $table_name,
 		FK_TABLE_SCHEM    => undef,
 		ORDINAL_POSITION  => $$row[1],
 		UK_COLUMN_NAME    => $$row[4],
 		UK_DATA_TYPE      => undef,
 		UK_NAME           => undef,
 		UK_TABLE_CAT      => undef,
-		UK_TABLE_NAME     => $table_name,
+		UK_TABLE_NAME     => $$row[2],
 		UK_TABLE_SCHEM    => undef,
 		UNIQUE_OR_PRIMARY => undef,
 		UPDATE_RULE       => $referential_action{$$row[5]},
@@ -867,9 +885,13 @@ The hashref returned for foreign keys contains these key-value pairs:
 
 This list of keys matches what is returned when processing a Postgres database.
 
-=head2 Haven't you got FK and PK backwards?
+=head2 Have you got FK and PK backwards?
 
-No, I don't think so.
+I certainly hope not. To me the FK_TABLE_NAME points to the UK_TABLE_NAME.
+
+The "pragma foreign_key_list($table_name)" call for SQLite returns data from the create statement, and thus it
+reports what the given table points to. The DBI call to foreign_key_info(...) returns data about foreign keys
+referencing (pointing to) the given table. This can be confusing.
 
 Here is a method from the module L<App::Office::Contacts::Util::Create>, part of L<App::Office::Contacts>.
 
